/**
 * Advanced Sidebar Menu PRO accordions.
 *
 * Widget, accordion functionality on front-end.
 *
 * @author OnPoint Plugins
 * @link https://onpointplugins.com/advanced-sidebar-menu/advanced-sidebar-menu-pro-widget-docs/advanced-sidebar-menu-pro-widget-accordions/
 *
 */
//phpcs:disable WordPressVIPMinimum.JS.HTMLExecutingFunctions

/* global asm_pro, asm_pro_config, jQuery */
var asm_pro = window.asm_pro || {
	enabled_accordions: {},
	accordion: {},
};
var ADVANCED_SIDEBAR_MENU_ACCORDIONS = window.ADVANCED_SIDEBAR_MENU_ACCORDIONS || {
	icons: {
		hamburger: {
			expand: '',
			contract: '',
		},
	},
	iconStyle: '',
};

( function( $, a, config ) {
	a.accordion = {
		/**
		 * Iterate through the registered accordions
		 * 1. Sets up the icons.
		 * 2. Hides the child menus.
		 *
		 * @param {boolean} force - To initialize the accordion even if already
		 *                         initialized. For edge cases and custom extensions.
		 *
		 */
		init: function( force = false ) {
			var self = a.accordion;
			var parents = [];

			for ( var widgetId in a.enabled_accordions ) {
				if ( ! a.enabled_accordions.hasOwnProperty( widgetId ) ) {
					continue;
				}

				var el = $( '[data-js="' + widgetId + '"]' );
				// Already enabled. Prevent icons from doubling up.
				if ( el.data( 'accordion-enabled' ) ) {
					// Reinitialize.
					if ( force ) {
						el.find( '[data-accordion-icon]' ).remove();
					} else {
						continue;
					}
				}

				// The .has() only target elements that have actual children in.
				// Case of 'always display child pages' not being active.
				if ( self.isChecked( widgetId, 'include_parent' ) && ! self.isChecked( widgetId, 'include_parent_in_accordion' ) ) {
					parents = el.find( '> ul > li .has_children' ).has( 'ul' );
				} else {
					parents = el.find( 'ul .has_children' ).has( 'ul' );
				}

				parents.each( function() {
					var $menuItem = $( this );
					var $icon;
					// Current page and parents of current page should be open.
					if ( ! self.isChecked( widgetId, 'keep_closed' )
						&& (
							$menuItem.hasClass( 'current_page_item' )
							|| $menuItem.hasClass( 'current-cat' )
							|| $menuItem.hasClass( 'current-menu-item' )
							|| $menuItem.find( '.current-cat' ).length
							|| $menuItem.find( '.current_page_item' ).length
							|| $menuItem.find( '.current-menu-item' ).length
						)
					) {
						$menuItem
							.addClass( 'open' )
							.find( '> ul' )
							.attr( 'aria-hidden', 'false' );
						$icon = self._get_icon( widgetId, 'contract', $menuItem );

						// Everything else should be closed.
					} else {
						$menuItem
							.addClass( 'closed' )
							.find( '> ul' )
							.attr( 'aria-hidden', 'true' )
							.hide();
						$icon = self._get_icon( widgetId, 'expand', $menuItem );
					}

					if ( ! $menuItem.find( '> span' ).length ) {
						$menuItem.find( '> a' ).append( $icon );
						self.attachLinkHandlers( widgetId, $menuItem );
					}
				} );

				// Set a data flag to know when accordions are active.
				// Use an attribute so users can see it in the HTML.
				el.attr( 'data-accordion-enabled', 1 );
			}
		},

		/**
		 * Icon was clicked. We are either going to expand
		 * or contract a menu based on the icon's state.
		 * After the menu is changed the icon will change
		 * to the new one here.
		 *
		 * @param {JQuery<HTMLLIElement>} $menuItem - Menu item we are working with.
		 * @param {JQuery<HTMLSpanElement>} $icon - Icon which was clicked.
		 *
		 * @private
		 */
		_expand_sub_menu: function( $menuItem, $icon ) {
			var newIcon;
			if ( $icon.data( 'state' ) === 'expand' ) {
				$menuItem
					.addClass( 'open' )
					.removeClass( 'closed' );
				$menuItem.find( '> ul' )
					.attr( 'aria-hidden', 'false' )
					.slideDown();
				newIcon = this._get_icon( $icon.data( 'widget_id' ), 'contract', $menuItem );
			} else {
				$menuItem
					.addClass( 'closed' )
					.removeClass( 'open' );
				$menuItem.find( '> ul' )
					.attr( 'aria-hidden', 'true' )
					.slideUp();
				newIcon = this._get_icon( $icon.data( 'widget_id' ), 'expand', $menuItem );
			}

			$icon.replaceWith( newIcon );
			newIcon.trigger( 'focus' );
		},


		/**
		 * Get open/close icon for the menu item.
		 *
		 * @param {string} widgetId - Id of this widget.
		 * @param {string} state - expand|contract.
		 * @param {JQuery<HTMLLIElement>} $menuItem - The menu item we are working with.
		 *
		 * @returns {JQuery<HTMLSpanElement>}
		 * @private
		 */
		_get_icon: function( widgetId, state, $menuItem ) {
			var style = a.enabled_accordions[ widgetId ][ config.iconStyle ];
			var $icon = $( config.icons[ style ][ state ] );
			$icon.data( 'state', state );
			$icon.data( 'widget_id', widgetId );
			$icon.attr( 'data-accordion-icon', 1 );

			return this.attachClickHandler( $icon, $menuItem, function() {
				return $icon;
			} );
		},

		/**
		 * Attach a click and keypress handler to an element to
		 * make it open/close a section of the accordion.
		 *
		 * This provided icon holds the state and replaces itself
		 * each time the state changes.
		 *
		 * @since 8.4.0
		 *
		 * @param {JQuery<HTMLSpanElement|HTMLLinkElement>} $el - Element to attach the handler.
		 * @param {JQuery<HTMLLIElement>}$menuItem - Item of the menu to open/close.
		 * @param {function() : JQuery<HTMLSpanElement>} getIcon - Icon of the menu item.
		 * @return {JQuery<HTMLSpanElement|HTMLLinkElement>}
		 */
		attachClickHandler: function( $el, $menuItem, getIcon ) {
			var expand = function( ev ) {
				// Don't trigger link handlers on click of span (prevents double-up).
				if ( 'SPAN' === ev.target.nodeName && 'A' === ev.currentTarget.nodeName ) {
					return;
				}
				ev.preventDefault();
				a.accordion._expand_sub_menu( $menuItem, getIcon() );
			};
			// Mouse clicked on element.
			$el.on( 'click', function( ev ) {
				expand( ev );
			} );
			// Enter key pressed by screen readers.
			$el.on( 'keypress', function( ev ) {
				if ( '13' === ( ev.keyCode ? ev.keyCode : ev.which ).toString() ) {
					expand( ev );
				}
			} );
			return $el;
		},

		/**
		 * Attach click handlers to links which match a level set to
		 * do so. If a click handler is attached, the link will no longer
		 * open it's URL on click, but instead open/close the accordion
		 *
		 * @param {string} widgetId - Id of widget.
		 * @param {JQuery<HTMLLIElement>} $menuItem - <li> menu item.
		 *
		 * @since 8.4.0
		 */
		attachLinkHandlers: function( widgetId, $menuItem ) {
			if ( ! this.isChecked( widgetId, 'links_expand' ) ) {
				return;
			}

			var menuLevel = parseInt( $menuItem.parent().data( 'level' ) );
			// Determine if this link is included based on menu level.
			var attach = a.enabled_accordions[ widgetId ][ 'links_expand_levels' ].some( function( level ) {
				switch ( true ) {
					case level === 'highest' && 0 === menuLevel:
					case level === 'child' && 1 === menuLevel:
					case level === 'grandchild' && 2 <= menuLevel:
					case level === 'all':
						return true;
				}
				return false;
			} );

			if ( attach ) {
				var link = $menuItem.find( '> a' );

				this.attachClickHandler( link, $menuItem, function() {
					return link.find( 'span' );
				} );
			}
		},


		/**
		 * Is a checkbox from a widget of a key checked.
		 *
		 * Done this way to allow for using asm_debug overrides to uncheck options.
		 *
		 * @param {string} $widget_id
		 * @param {string} option
		 * @returns {*|boolean}
		 */
		isChecked: function( $widget_id, option ) {
			return a.enabled_accordions[ $widget_id ][ option ] && a.enabled_accordions[ $widget_id ][ option ] === 'checked';
		},

	};

	$( function() {
		a.accordion.init();

		setTimeout( function() {
			// Enables accordions when editing Elementor.
			if ( 'undefined' !== typeof elementorFrontend ) {
				elementorFrontend.hooks.addAction( 'frontend/element_ready/widget', function() {
					a.accordion.init();
				} );
				// #9064 - Elementor popups require icon regeneration.
				$( document ).on( 'elementor/popup/show', function() {
					a.accordion.init( true );
				} );
			}

			// Enables accordions when editing Beaver Builder.
			$( '.fl-builder-content' ).on( 'fl-builder.layout-rendered', function() {
				a.accordion.init();
			} );
		}, 1000 );

		$( document ).trigger( 'advanced-sidebar-menu-pro/accordions/loaded', [ a.accordion, a ] );
	} );

	$( document ).trigger( 'advanced-sidebar-menu-pro/accordions/available', [ a.accordion, a ] );
} )( jQuery, asm_pro, ADVANCED_SIDEBAR_MENU_ACCORDIONS );
