<?php

namespace Advanced_Sidebar_Menu\Meta;

use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget_Options\Page\Post_Type;

/**
 * Meta-box for title and exclude settings.
 */
class Title {
	use Singleton;

	const CACHE        = 'advanced-sidebar-menu-pro/excluded-pages';
	const EXCLUDE_PAGE = 'advanced-sidebar-menu-pro/exclude-this-page';
	const LINK_TEXT    = 'asm-link-text';
	const NONCE        = 'advanced-sidebar-menu-pro/title-nonce';


	/**
	 * Actions.
	 */
	protected function hook() {
		add_action( 'do_meta_boxes', [ $this, 'register_meta_box' ] );
		add_action( 'save_post', [ $this, 'save' ] );
		add_action( 'advanced-sidebar-menu/widget/before-render', [ $this, 'start_filter' ] );
		add_action( 'advanced-sidebar-menu/widget/after-render', [ $this, 'remove_filter' ] );
		add_filter( 'advanced-sidebar-menu/debug/print-instance', [ $this, 'add_exclude_pages_to_debug_data' ] );
		add_filter( 'advanced-sidebar-menu/menus/page/excluded', [ $this, 'add_excluded_by_meta_box_pages' ], 9 );
		add_action( 'advanced-sidebar-menu/widget/page/exclude-box', [ $this, 'mention_exclude_settings' ] );
	}


	/**
	 * Filter the title only in the context of this widget.
	 *
	 * @see Title::remove_filter()
	 */
	public function start_filter() {
		add_filter( 'the_title', [ $this, 'change_link_title' ], 99, 2 );
	}


	/**
	 * Filter the title only in the context of this widget.
	 *
	 * @see Title::add_filter()
	 */
	public function remove_filter() {
		remove_filter( 'the_title', [ $this, 'change_link_title' ], 99 );
	}


	/**
	 * Register the meta box on all post types supported by this plugin.
	 *
	 * @see Post_Type::get_post_types()
	 *
	 * @return void
	 */
	public function register_meta_box() {
		array_map( function ( $post_type ) {
			add_meta_box( 'asm-custom-title', esc_html__( 'Advanced Sidebar Menu', 'advanced-sidebar-menu' ), [
				$this,
				'render',
			], $post_type, 'side', 'high' );
		}, array_keys( Post_Type::instance()->get_post_types() ) );
	}


	/**
	 * Get all page ids, which are excluded based on the meta box setting.
	 *
	 * @since 3.11.0
	 *
	 * @return array
	 */
	public function get_exclude_page_ids() {
		$ids = wp_cache_get( self::CACHE );
		if ( false === $ids ) {
			$ids = get_posts(
				[
					'post_type'              => \array_keys( Post_Type::instance()->get_post_types() ),
					'nopaging'               => true,
					'fields'                 => 'ids',
					'update_post_meta_cache' => false,
					'update_post_term_cache' => false,
					'suppress_filters'       => false,
					//phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
					'meta_query'             => [
						[
							'key'     => self::EXCLUDE_PAGE,
							'compare' => 'EXISTS',
						],
					],
				]
			);
			if ( empty( $ids ) ) {
				$ids = [];
			} else {
				$ids = \array_map( 'intval', $ids );
			}
			wp_cache_set( self::CACHE, $ids );
		}

		return (array) $ids;
	}


	/**
	 * List the excluded pages withing debug data.
	 *
	 * @param array $data - Data before widget settings are added.
	 *
	 * @since 8.6.0
	 */
	public function add_exclude_pages_to_debug_data( array $data ) {
		$data['excluded_pages'] = $this->get_exclude_page_ids();

		return $data;
	}


	/**
	 * Add the pages excluded by meta settings to pages excluded via widget settings.
	 *
	 * @param array $excluded Excluded via widget settings.
	 *
	 * @return array
	 */
	public function add_excluded_by_meta_box_pages( $excluded ) {
		return \array_unique( \array_merge( $excluded, $this->get_exclude_page_ids() ) );
	}


	/**
	 * Use the value set in the meta box for the title.
	 *
	 * @param string $title     - the original title.
	 * @param ?int   $post_id   - A default value is required because some
	 *                          plugins don't pass a second argument.
	 *
	 * @see https://onpointplugins.com/plugin-support/plugin-conflict-with-nextend-smart-slider-plugin/
	 *
	 * @return mixed
	 */
	public function change_link_title( $title, $post_id = null ) {
		$text = get_post_meta( $post_id, self::LINK_TEXT, true );
		if ( empty( $text ) ) {
			return $title;
		}

		return $text;
	}


	/**
	 * Save the meta box.
	 *
	 * @param int $post_id - ID of the post we're saving.
	 */
	public function save( $post_id ) {
		if ( ! empty( $_POST[ self::NONCE ] ) && wp_verify_nonce( $_POST[ self::NONCE ], self::NONCE ) ) {
			if ( empty( $_POST[ self::LINK_TEXT ] ) ) {
				delete_post_meta( $post_id, self::LINK_TEXT );
			} else {
				update_post_meta( $post_id, self::LINK_TEXT, sanitize_text_field( wp_unslash( $_POST[ self::LINK_TEXT ] ) ) );
			}

			if ( empty( $_POST[ self::EXCLUDE_PAGE ] ) ) {
				delete_post_meta( $post_id, self::EXCLUDE_PAGE );
			} else {
				update_post_meta( $post_id, self::EXCLUDE_PAGE, 1 );
			}

			$this->flush_cache();
		}
	}


	/**
	 * Render the meta-box content.
	 *
	 * @param \WP_Post $post - The post we are editing.
	 */
	public function render( \WP_Post $post ) {
		wp_nonce_field( self::NONCE, self::NONCE );
		$post_type = get_post_type_object( $post->post_type );
		if ( null === $post_type ) {
			$post_type = get_post_type_object( 'page' ); // Sensible default.
		}
		?>
		<p>
			<?php
			/* translators: Current post type single label */
			printf( esc_html__( "Title used for this %s's link when it is displayed in the Advanced Sidebar menus.", 'advanced-sidebar-menu' ), esc_html( strtolower( $post_type->labels->singular_name ) ) );
			?>
			<label class="screen-reader-text" for="<?php echo esc_attr( self::LINK_TEXT ); ?>">
				<?php
				/* translators: Current post type single label */
				printf( esc_html__( "%s widget link's text", 'advanced-sidebar-menu' ), esc_html( $post_type->labels->singular_name ) );
				?>
			</label>
			<input
				id="<?php echo esc_attr( self::LINK_TEXT ); ?>"
				name="<?php echo esc_attr( self::LINK_TEXT ); ?>"
				type="text"
				class="widefat"
				value="<?php echo esc_attr( get_post_meta( $post->ID, self::LINK_TEXT, true ) ); ?>"
				placeholder="<?php echo esc_attr( $post->post_title ); ?>">

		</p>
		<p>
			<label class="screen-reader-text" for="<?php echo esc_attr( self::EXCLUDE_PAGE ); ?>">
				<?php
				/* translators: Current post type single label */
				printf( esc_html__( 'Exclude this %s from all Advanced Sidebar menus.', 'advanced-sidebar-menu' ), esc_html( $post_type->labels->singular_name ) );
				?>
			</label>
			<input
				name="<?php echo esc_attr( self::EXCLUDE_PAGE ); ?>" type="checkbox" value="1"
				id="<?php echo esc_attr( self::EXCLUDE_PAGE ); ?>"
				<?php checked( (bool) get_post_meta( $post->ID, self::EXCLUDE_PAGE, true ) ); ?>
			/>
			<?php
			/* translators: Current post type single label */
			printf( esc_html__( 'Exclude this %s from all Advanced Sidebar menus.', 'advanced-sidebar-menu' ), esc_html( strtolower( $post_type->labels->singular_name ) ) );
			?>
		</p>
		<?php
	}


	/**
	 * Mention excluding a page by its individual page edit
	 * settings.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @since 8.1.3
	 */
	public function mention_exclude_settings( array $instance = [] ) {
		?>
		<span class="description">
			<em>
				<?php
				/* translators: {post type plural label}, {post type singe label}, {<a>}, {</a>} */
				printf( esc_html__( '%1$s may also be excluded from all menus via the Advanced Sidebar Menu settings when %3$sediting a %2$s%4$s.', 'advanced-sidebar-menu' ),
					esc_html( Post_Type::instance()->get_post_type_label( $instance, false ) ),
					esc_html( strtolower( Post_Type::instance()->get_post_type_label( $instance ) ) ),
					'<a href="https://onpointplugins.com/advanced-sidebar-menu/advanced-sidebar-menu-pro-widget-docs/#customize-link-title" target="_blank">',
					'</a>'
				);
				?>
			</em>
		</span>
		<?php
	}


	/**
	 * Delete the excluded ids from the cache.
	 */
	protected function flush_cache() {
		wp_cache_delete( self::CACHE );
	}
}
