<?php

namespace Advanced_Sidebar_Menu\Nav_Menu;

use Advanced_Sidebar_Menu\Meta\Title;

/**
 * Nav Menu Objects
 *
 * @author OnPoint Plugins
 * @since  3.3.3
 */
class Objects {
	/**
	 * All WP nav menu items.
	 *
	 * @var Item[]
	 */
	public $menu_items = [];

	/**
	 * Menu items broken out into single level array of parent => child
	 *
	 * @var array
	 */
	public $hierarchy = [];


	/**
	 * Get a nav menu item.
	 *
	 * @param int $post_id - ID of the nav menu post type.
	 *
	 * @return Item|bool
	 */
	public function get_item( $post_id ) {
		if ( ! isset( $this->menu_items[ $post_id ] ) ) {
			return false;
		}

		return $this->menu_items[ $post_id ];
	}


	/**
	 * If we only want the top level items
	 *
	 * @return void
	 */
	public function remove_all_child_items() {
		foreach ( $this->menu_items as $_id => $_item ) {
			if ( 0 !== (int) $_item->menu_item_parent ) {
				unset( $this->menu_items[ $_id ] );
			}
		}
	}


	/**
	 * If we only want items which are part of
	 * the current item's hierarchy.
	 *
	 * @return void
	 */
	public function remove_all_non_family_items() {
		foreach ( $this->menu_items as $_id => $_item ) {
			if ( ! $this->is_in_current_family( $_item->get_id() ) ) {
				unset( $this->menu_items[ $_id ] );
			}
		}
	}


	/**
	 * Is this post a parent, child, or grandchild of
	 * the current page's top-level parent.
	 *
	 * @param int $post_id - The post we are checking against.
	 *
	 * @return bool
	 */
	public function is_in_current_family( $post_id ) {
		$family = $this->get_current_item_family();

		return in_array( (int) $post_id, $family, true );
	}


	/**
	 * Get all items which are the current page's section based on the
	 * top-level parent of the current page.
	 *
	 * Get all ids which are children or grandchildren of the current
	 * page's top-level parent.
	 *
	 * @return int[]
	 */
	public function get_current_item_family() {
		static $current_family;
		if ( null !== $current_family ) {
			return $current_family;
		}

		$top = $this->get_current_top_level_parent();
		if ( empty( $top ) ) {
			return [];
		}

		$children = $this->get_child_items_recursive( $top->get_id() );
		$current_family = array_map( [ $this, 'flatten_to_id' ], $children );
		return $current_family;
	}


	/**
	 * Get all items which are direct children of a nav item
	 * by menu item ID.
	 *
	 * @param int $post_id - ID of the menu item post type.
	 *
	 * @return Item[]
	 */
	public function get_child_items( $post_id ) {
		$items = [];
		if ( ! empty( $this->hierarchy[ $post_id ] ) ) {
			foreach ( $this->hierarchy[ $post_id ] as $_child ) {
				$items[ $_child ] = $this->menu_items[ $_child ];
			}
		}

		return $items;
	}


	/**
	 * Get all parents and grandparents.
	 *
	 * @param Item $item - Item we want parents of.
	 *
	 * @since 3.11.0
	 * @return Item[]
	 */
	public function get_parent_items_recursive( Item $item ) {
		$parents = [];
		$parent = $item->get_parent();
		while ( false !== $parent ) {
			$parents[] = $parent;
			$parent = $parent->get_parent();
		}

		return $parents;
	}


	/**
	 * Get top-level parent based on the current item.
	 *
	 * @return Item|bool
	 */
	public function get_current_top_level_parent() {
		$current = $this->get_current_item();
		if ( ! empty( $current ) ) {
			while ( false !== $current->get_parent() ) {
				$current = $current->get_parent();
			}
		}

		return $current;
	}


	/**
	 * Get WP menu item which matches the current page we are viewing.
	 *
	 * @return bool|Item
	 */
	public function get_current_item() {
		$current = wp_filter_object_list( $this->menu_items, [ 'current' => true ] );
		$post = reset( $current );

		return empty( $post ) ? false : $post;
	}


	/**
	 * Put menu items back into a list that `wp_nav_menu` can understand.
	 *
	 * @return \WP_Post[]
	 */
	public function flatten() {
		$menu_items = [];
		foreach ( $this->menu_items as $_item ) {
			if ( ! $_item->is_excluded() ) {
				$menu_items[] = $_item->get_menu_item();
			}
		}

		return $menu_items;
	}


	/**
	 * Get the current hierarchy of the menu.
	 *
	 * @internal
	 *
	 * @return array
	 */
	public function get_hierarchy() {
		return $this->hierarchy;
	}


	/**
	 * Get the ID of this menu item.
	 *
	 * @param Item $item - The menu item to get the ID of.
	 *
	 * @return int
	 */
	protected function flatten_to_id( $item ) {
		return $item->get_id();
	}


	/**
	 * Get all the child, grandchild, self, great grandchild etc.
	 *
	 * @param int $post_id - ID of menu item we want children of.
	 *
	 * @return Item[]
	 */
	public function get_child_items_recursive( $post_id ) {
		if ( empty( $this->menu_items[ $post_id ] ) ) {
			return [];
		}
		$items = [ [ $this->menu_items[ $post_id ] ] ];
		if ( ! empty( $this->hierarchy[ $post_id ] ) ) {
			foreach ( $this->hierarchy[ $post_id ] as $_child_id ) {
				$items[] = $this->get_child_items_recursive( $_child_id );
			}
		}

		return call_user_func_array( 'array_merge', $items );
	}


	/**
	 * Put menu items into parent/child structure.
	 *
	 * @return void
	 */
	protected function parse_hierarchy() {
		foreach ( $this->menu_items as $_item ) {
			$this->hierarchy[ (int) $_item->menu_item_parent ][] = $_item->get_id();
		}

		do_action( 'advanced-sidebar-menu/menus/navigation/objects/parse-hierarchy', $this->hierarchy, $this->get_current_item(), $this, $this->menu_items );
	}


	/**
	 * Get all exclude navigation items by id.
	 *
	 * The ID is the navigation item, not the object/post ID.
	 * This retrieves the page ids from the title meta box and translates
	 * them to menu item IDs.
	 *
	 * @since 3.11.0
	 *
	 * @return array
	 */
	public function get_excluded_ids() {
		static $ids = null;
		if ( null !== $ids ) {
			return $ids;
		}
		$ids = [];
		$excluded_pages = Title::instance()->get_exclude_page_ids();
		// Translate page ids to nav menu ids.
		foreach ( $this->menu_items as $_item ) {
			if ( $_item->is_page() && in_array( $_item->get_object_id(), $excluded_pages, true ) ) {
				$ids[] = $_item->get_id();
			}
		}

		return array_map( 'intval', apply_filters( 'advanced-sidebar-menu/menus/navigation/objects/excluded', $ids, $this->get_current_item(), $this, $this->menu_items, $this->hierarchy ) );

	}


	/**
	 * Create an instance by providing a menu's items.
	 *
	 * @param \WP_Post[] $menu_items - The menus items.
	 *
	 * @static
	 *
	 * @return Objects
	 */
	public static function factory( array $menu_items ) {
		/* @phpstan-ignore-next-line */
		$objects = new static();
		foreach ( $menu_items as $_item ) {
			$objects->menu_items[ (int) $_item->ID ] = Item::factory( $_item, $objects );
		}
		$objects->parse_hierarchy();

		return $objects;
	}

}
