<?php

namespace OnPoint_Plugins\Pro_Plugins;

if ( ! \class_exists( __NAMESPACE__ . '\Update' ) ) {
	/**
	 * Update the plugin from the api.
	 *
	 * @author OnPoint Plugins
	 */
	final class Update {
		const VERSION = '4.2.3';

		/**
		 * Root directory of this plugin.
		 *
		 * @var string;
		 */
		private $root;

		/**
		 * Slug of this plugin.
		 *
		 * @var string
		 */
		private $slug;

		/**
		 * This plugin's version.
		 *
		 * @var string
		 */
		private $version;

		/**
		 * Store subsequent responses to prevent extra traffic
		 * to our server.
		 *
		 * @var array
		 */
		private $cache = [];


		/**
		 * Create the instance of the class.
		 *
		 * @param string $slug    - Plugin slug.
		 * @param string $root    - Plugin root directory.
		 * @param string $version - This plugin's version.
		 */
		public function __construct( $slug, $root, $version ) {
			$this->slug = $slug;
			$this->root = $root;
			$this->version = $version;

			$this->hook();
		}


		/**
		 * Add actions and filters.
		 */
		private function hook() {
			add_filter( 'pre_set_site_transient_update_plugins', [ $this, 'check_for_update' ] );
			add_filter( 'plugins_api', [ $this, 'get_plugin_info' ], 10, 3 );
			add_action( "after_plugin_row_{$this->slug}/{$this->slug}.php", [
				$this,
				'invalid_license_row',
			], 9, 2
			);
			add_action( 'all_admin_notices', [ $this, 'invalid_license_notice' ] );

			if ( defined( 'ONPOINT_PLUGINS_API_CHECK_VERSION' ) ) {
				delete_site_transient( 'update_plugins' );
			}

			add_action( 'plugins_loaded', function () {
				do_action( "{$this->slug}/updates/loaded", $this->get_invalid_message() );
			}, 20 );
		}


		/**
		 * Get the current version of this plugin.
		 *
		 * @since 2.1.0
		 *
		 * @return mixed|null
		 */
		private function get_current_version() {
			if ( defined( 'ONPOINT_PLUGINS_API_CHECK_VERSION' ) ) {
				return ONPOINT_PLUGINS_API_CHECK_VERSION;
			}

			return $this->version;
		}


		/**
		 * Display an admin notice if the current license is invalid.
		 *
		 * @since 2.2.0
		 *
		 * @return void
		 */
		public function invalid_license_notice() {
			$message = $this->get_invalid_message();
			if ( $message ) {
				?>
				<div class="error">
					<p>
						<?php echo $message; //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
					</p>
				</div>
				<?php
			}
		}


		/**
		 * If the license is invalid this displays a notice in the plugins list.
		 *
		 * @param string $plugin_file - Slug and main plugin file.
		 * @param array  $plugin_data - Accumulated data from plugin file and endpoint.
		 *
		 * @since 2.2.0
		 *
		 * @return void
		 */
		public function invalid_license_row( $plugin_file, $plugin_data ) {
			if ( isset( $plugin_data['invalid_license'] ) ) {
				// Remove default update message.
				remove_action( "after_plugin_row_{$plugin_file}", 'wp_plugin_update_row' );

				if ( is_network_admin() ) {
					$active_class = is_plugin_active_for_network( $plugin_file ) ? ' active' : '';
				} else {
					$active_class = is_plugin_active( $plugin_file ) ? ' active' : '';
				}
				?>
				<tr class="plugin-update-tr<?php echo esc_attr( $active_class ); ?>">
					<td colspan="4" class="plugin-update">
						<div class="update-message notice inline notice-error">
							<p>
								<?php echo $plugin_data['invalid_license']; //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
							</p>
						</div>
					</td>
				</tr>
				<?php
			}
		}


		/**
		 * Checks our custom location for an available update
		 *
		 * @param ?object $plugins - Date about all the current plugins and their version.
		 *
		 * @return object
		 */
		public function check_for_update( $plugins ) {
			if ( empty( $plugins ) ) {
				$plugins = new \stdClass();
			}
			$version = $this->get_current_version();
			$plugins->checked[ $this->slug . '/' . $this->slug . '.php' ] = $version;
			$args = [
				'slug'    => $this->slug,
				'version' => $version,
			];
			$response = $this->do_request( $args, 'basic_check' );

			if ( ! empty( $response ) && \is_object( $response ) ) {
				if ( version_compare( $version, $response->new_version, '<' ) ) {
					$plugins->response[ $this->slug . '/' . $this->slug . '.php' ] = $response;
				} else {
					$plugins->no_update[ $this->slug . '/' . $this->slug . '.php' ] = $response;
				}
			}

			return $plugins;
		}


		/**
		 * Point any plugin api calls which match this plugin's slug
		 * to our custom endpoint.
		 *
		 * @param mixed  $info   - Typically this is just `false` if not filtered.
		 * @param string $action - Action we are calling.
		 * @param object $args   - Plugin arguments.
		 *
		 * @since 2.0.0
		 *
		 * @return ?object
		 */
		public function get_plugin_info( $info, $action, $args ) {
			if ( empty( $args->slug ) || $this->slug !== $args->slug ) {
				return $info;
			}

			$data = (array) $args;
			$data['version'] = $this->get_current_version();

			return $this->do_request( $data, $action );
		}


		/**
		 * Retrieve the license info from the `LICENSE.php` file.
		 *
		 * @since 2.0.0
		 *
		 * @return array
		 */
		private function get_license() {
			$license = include $this->root . '/LICENSE.php';
			if ( defined( 'ONPOINT_PLUGINS_API_LICENSE' ) ) {
				$license['hash'] = ONPOINT_PLUGINS_API_LICENSE;
			}

			return $license;
		}


		/**
		 * Retrieve the url of the update api.
		 *
		 * @since 2.2.0
		 *
		 * @return string
		 */
		private function get_api_url() {
			if ( defined( 'ONPOINT_PLUGINS_API_URL' ) ) {
				return ONPOINT_PLUGINS_API_URL;
			}

			return 'https://onpointplugins.com/plugins/v2/';
		}


		/**
		 * Return the error message for an invalid license
		 * if applicable. May also be used to check validity
		 * of license.
		 *
		 * Returns false if license is valid.
		 *
		 * @since 4.1.0
		 *
		 * @return ?string
		 */
		private function get_invalid_message() {
			$plugins = get_site_transient( 'update_plugins' );

			// An update is available, and invalid license.
			if ( isset( $plugins->response[ $this->slug . '/' . $this->slug . '.php' ]->invalid_license ) ) {
				return $plugins->response[ $this->slug . '/' . $this->slug . '.php' ]->invalid_license;
			}

			// An update is not available, yet invalid license.
			if ( isset( $plugins->no_update[ $this->slug . '/' . $this->slug . '.php' ]->invalid_license ) ) {
				return $plugins->no_update[ $this->slug . '/' . $this->slug . '.php' ]->invalid_license;
			}

			return null;
		}


		/**
		 * Make a POST request to the plugin api endpoint.
		 *
		 * Subsequent requests during the same page load will receive a
		 * cached result based on action.
		 *
		 * @param array  $args   - Arguments to pass as the `request` parameter.
		 * @param string $action - The action we are executing.
		 *
		 * @return ?object
		 */
		private function do_request( $args, $action ) {
			if ( ! empty( $this->cache[ $action ] ) ) {
				return $this->cache[ $action ];
			}
			$request = [
				'body'       => [
					'action'           => $action,
					'args'             => wp_json_encode( $args ),
					'origin'           => get_bloginfo( 'url' ),
					'license'          => $this->get_license(),
					'endpoint_version' => self::VERSION,
				],
				'user-agent' => 'WordPress/' . get_bloginfo( 'version' ) . '; ' . get_bloginfo( 'url' ),
			];

			$raw_response = wp_remote_post( $this->get_api_url(), $request );
			if ( is_wp_error( $raw_response ) || 200 !== wp_remote_retrieve_response_code( $raw_response ) ) {
				return null;
			}

			$data = json_decode( wp_remote_retrieve_body( $raw_response ), true );
			if ( empty( $data ) ) {
				return null;
			}

			// Cache response to prevent duplicate requests.
			$this->cache[ $action ] = (object) $data;

			return $this->cache[ $action ];
		}

	}
}
