<?php

namespace Advanced_Sidebar_Menu\Walkers;

use Advanced_Sidebar_Menu\Menus\Category;
use Advanced_Sidebar_Menu\Meta\Title;
use Advanced_Sidebar_Menu\Widget_Options\Category\Display_Posts;

/**
 * Add posts under each category if the option is selected to do so.
 */
class Posts extends Category_Walker {
	/**
	 * The widget instance
	 *
	 * @var array
	 */
	public $instance;

	/**
	 * Level of menu this is going under.
	 *
	 * @var string
	 */
	public $level;

	/**
	 * Posts assigned to the category
	 *
	 * @var array
	 */
	protected $children;


	/**
	 * Posts constructor.
	 *
	 * @param array  $instance - Current widget settings.
	 * @param string $level    - Level of menu this is going under.
	 */
	public function __construct( array $instance, $level = Category::LEVEL_DISPLAY_ALL ) {
		$this->instance = $instance;
		$this->level = $level;
	}


	/**
	 * Add the has_children class to an item which has post children
	 *
	 * @param array $css_classes - Current item classes.
	 *
	 * @return array
	 */
	public function add_has_children_class( $css_classes ) {
		$css_classes[] = 'has_children';

		return $css_classes;
	}


	/**
	 * Add the current-cat-parent class to items, which have the current
	 * post as a child post.
	 *
	 * @param array $css_classes - Current item classes.
	 *
	 * @return array
	 */
	public function add_current_cat_parent_class( $css_classes ) {
		$css_classes[] = 'current-cat-parent';

		return $css_classes;
	}


	/**
	 * Get child posts of term based on passed widget settings.
	 *
	 * When display-all is checked we need to catch the cases where a level
	 * should not be included here because the walker will be added to every
	 * level due to the single call to `wp_list_categories()`.
	 *
	 * @param \WP_Term $term  - Current category or term.
	 * @param int      $depth - What level of the menu we are on.
	 * @param array    $args  - Additional arguments passed to wp_list_categories.
	 * @param int      $id    - ID of the current item.
	 *
	 * @since 3.5.0
	 *
	 * @return array
	 */
	public function get_children( \WP_Term $term, $depth, $args, $id ) {
		$_args = apply_filters( 'advanced-sidebar-menu-pro/walker/display-posts/get-post-args', [
			'post_type'        => $this->instance[ Display_Posts::POST_TYPE ],
			'numberposts'      => $this->instance[ Display_Posts::LIMIT ],
			'orderby'          => 'title',
			'order'            => 'ASC',
			'suppress_filters' => false,
			//phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_tax_query
			'tax_query'        => [
				[
					'taxonomy'         => $term->taxonomy,
					'terms'            => $term->term_id,
					'include_children' => false,
				],
			],
		], $term, $depth, $args, $id, $this );

		// display-all is checked, so we have to handle the different levels here.
		if ( ! empty( $this->instance[ Category::DISPLAY_ALL ] ) ) {
			$levels = $this->instance[ Display_Posts::NAME ];
			switch ( $levels ) {
				case Display_Posts::LEVELS_PARENT:
					if ( 0 < $depth || Category::LEVEL_PARENT !== $args['level'] ) {
						return [];
					}
					break;
				case Display_Posts::LEVELS_PARENT_AND_CHILD:
					if ( 0 < $depth || ( Category::LEVEL_PARENT !== $args['level'] && Category::LEVEL_DISPLAY_ALL !== $args['level'] ) ) {
						return [];
					}
					break;
				case Display_Posts::LEVELS_CHILD:
					if ( 0 < $depth || Category::LEVEL_DISPLAY_ALL !== $args['level'] ) {
						return [];
					}
					break;
				case Display_Posts::LEVELS_CHILD_AND_GRANDCHILD:
					if ( Category::LEVEL_DISPLAY_ALL !== $args['level'] ) {
						return [];
					}
					break;
				case Display_Posts::LEVELS_GRANDCHILD:
					if ( 1 > $depth || Category::LEVEL_DISPLAY_ALL !== $args['level'] ) {
						return [];
					}
					break;
			}
		}

		$excluded = $this->get_excluded_ids();
		return \array_values( \array_filter( get_posts( $_args ), function ( $post ) use ( $excluded ) {
			return ! \in_array( $post->ID, $excluded, true );
		} ) );
	}


	/**
	 * Add the necessary filters and retrieve posts for the current list item.
	 * Then call the parent `start_el` like normal.
	 *
	 * @param string   $output - Current list output.
	 * @param \WP_Term $term   - Current category or term.
	 * @param int      $depth  - What level of the menu we are on.
	 * @param array    $args   - Additional arguments passed to wp_list_categories.
	 * @param int      $id     - Id of the current item.
	 *
	 * @return void
	 */
	public function start_el( &$output, $term, $depth = 0, $args = [], $id = 0 ) {
		$this->children[ $term->term_id ] = $this->get_children( $term, $depth, $args, $id );

		if ( ! empty( $this->children[ $term->term_id ] ) ) {
			add_filter( 'category_css_class', [ $this, 'add_has_children_class' ] );
			// add the current-cat-parent class to the parent term of the current post.
			$current_post = $this->get_current_post();
			if ( null !== $current_post ) {
				$ids = wp_list_pluck( $this->children[ $term->term_id ], 'ID' );
				if ( in_array( $current_post->ID, $ids, false ) ) { //phpcs:ignore
					add_filter( 'category_css_class', [ $this, 'add_current_cat_parent_class' ] );
				}
			}
		}

		parent::start_el( $output, $term, $depth, $args, $id );
	}


	/**
	 * Ends the element output, if needed.
	 *
	 * @param string   $output Used to append additional content (passed by reference).
	 * @param \WP_Term $term   Current category.
	 * @param int      $depth  Optional. Depth of category. Not used.
	 * @param array    $args   Optional. An array of arguments. Only uses 'list' for whether should append
	 *                         to output. See wp_list_categories(). Default empty array.
	 */
	public function end_el( &$output, $term, $depth = 0, $args = [] ) {
		remove_filter( 'category_css_class', [ $this, 'add_has_children_class' ] );
		remove_filter( 'category_css_class', [ $this, 'add_current_cat_parent_class' ] );
		$current_post = $this->get_current_post();

		$level = 0;
		switch ( $this->level ) {
			case Category::LEVEL_DISPLAY_ALL:
				$level = $depth + 2;
				break;
			case Category::LEVEL_PARENT:
				$level = 1;
				break;
			case Category::LEVEL_CHILD:
				$level = 2;
				break;
			case Category::LEVEL_GRANDCHILD:
				$level = 3;
				break;
		}

		if ( ! empty( $this->children[ $term->term_id ] ) ) {
			ob_start();
			?>
			<ul class="children children-posts" data-level="<?php echo esc_attr( (string) $level ); ?>">
				<?php
				foreach ( $this->children[ $term->term_id ] as $_post ) {
					$class = "child-post cat-item {$_post->post_type}-item-{$_post->ID}";
					if ( 'post' !== $_post->post_type ) {
						$class .= " child-{$_post->post_type}";
					}
					if ( null !== $current_post && $current_post->ID === $_post->ID ) {
						$class .= ' current-cat';
					}
					?>
					<li class="<?php echo esc_attr( $class ); ?>">
						<a
							href="<?php echo esc_url( get_permalink( $_post ) ); ?>"
							title="<?php echo esc_attr( $_post->post_title ); ?>">
							<?php echo esc_html( get_the_title( $_post ) ); ?>
						</a>
						<?php
						// @link https://onpointplugins.com/plugin-support/show-child-posts-under-parent-posts/
						do_action( 'advanced-sidebar-menu-pro/walker/display-posts/post-element', $_post, $this->instance, $this );
						?>
					</li>
					<?php
				}
				?>
			</ul>
			<?php
			$output .= ob_get_clean();
		}

		parent::end_el( $output, $term, $depth, $args );
	}


	/**
	 * Get the post ids excluded from the page meta box.
	 *
	 * @since 8.6.0
	 *
	 * @return int[]
	 */
	public function get_excluded_ids() {
		return apply_filters( 'advanced-sidebar-menu-pro/walker/posts/excluded-ids', Title::instance()->get_exclude_page_ids(), $this );
	}


	/**
	 * Get the current post we are current viewing which matches the post
	 * type setting for this widget.
	 *
	 * If we are not on a single post/page/cpt returns null.
	 *
	 * @since 3.8.4
	 *
	 * @return \WP_Post|null
	 */
	protected function get_current_post() {
		if ( empty( $this->instance[ Display_Posts::POST_TYPE ] ) ) {
			$this->instance[ Display_Posts::POST_TYPE ] = 'post';
		}
		if ( is_singular( $this->instance[ Display_Posts::POST_TYPE ] ) ) {
			return get_post();
		}
		return null;
	}
}
