<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Category;

use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Display Everywhere Option for Category Widget
 *
 * @author OnPoint Plugins
 * @since  3.1.0
 */
class Display_Everywhere extends Widget_Options_Abstract {
	use Singleton;

	const NAME            = 'display_everywhere';
	const PARENT_CATEGORY = 'display_everywhere/parent_category';


	/**
	 * Actions and filters.
	 */
	protected function hook() {
		add_filter( 'advanced-sidebar-menu/menus/category/is-displayed', [ $this, 'is_displayed_always' ], 10, 4 );
		add_filter( 'advanced-sidebar-menu/menus/category/included-term-ids', [
			$this,
			'filter_top_level_terms',
		], 10, 4 );

		add_action( 'advanced-sidebar-menu/widget/category/right-column', [ $this, 'render_fields' ], 5, 2 );
	}


	/**
	 * Is this widget set to always display?
	 *
	 * @param bool  $displayed   - Is this menu being displayed.
	 * @param array $widget_args - Registered widget arguments.
	 * @param array $instance    - Widget settings.
	 *
	 * @return bool
	 */
	public function is_displayed_always( $displayed, array $widget_args, array $instance ) {
		if ( $this->checked( self::NAME, $instance ) ) {
			return true;
		}

		return $displayed;
	}


	/**
	 * Filter the top-level terms for this widget based on these settings.
	 *
	 * @param array $term_ids    - List of the highest level terms.
	 * @param array $widget_args - Registered widget arguments.
	 * @param array $instance    - Widget settings.
	 *
	 * @return array|int|\WP_Error
	 */
	public function filter_top_level_terms( $term_ids, array $widget_args, array $instance ) {
		if ( $this->checked( self::NAME, $instance ) ) {
			// Specified one only.
			if ( ! empty( $instance[ self::PARENT_CATEGORY ] ) ) {
				return [ (int) $instance[ self::PARENT_CATEGORY ] ];
			}

			// All top level terms.
			return get_terms( [
				'parent'     => 0,
				'fields'     => 'ids',
				'taxonomy'   => Taxonomy::instance()->get_taxonomy( 'category', [], $instance ),
				'hide_empty' => true,
			] );
		}

		return $term_ids;
	}


	/**
	 * Get the current taxonomy.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @return string
	 * @deprecated
	 */
	protected function get_taxonomy( array $instance ) {
		_deprecated_function( __METHOD__, '8.2.0', esc_html( Taxonomy::class . '::get_taxonomy' ) );
		return Taxonomy::instance()->get_taxonomy( 'category', [], $instance );
	}


	/**
	 * Get the label of the current taxonomy.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @return string
	 * @deprecated
	 */
	public function get_taxonomy_label( array $instance ) {
		_deprecated_function( __METHOD__, '8.2.0', esc_html( Taxonomy::class . '::get_taxonomy_label' ) );
		return Taxonomy::instance()->get_taxonomy_label( $instance );
	}


	/**
	 * Get the plural version of taxonomy label.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @return string
	 * @deprecated
	 */
	public function get_taxonomy_label_plural( array $instance ) {
		_deprecated_function( __METHOD__, '8.2.0', esc_html( Taxonomy::class . '::get_taxonomy_label' ) );
		return Taxonomy::instance()->get_taxonomy_label( $instance, false );
	}


	/**
	 * Render the fields in the widget.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance[ self::NAME ] = isset( $instance[ self::NAME ] ) ? $instance[ self::NAME ] : '';
		$instance[ self::PARENT_CATEGORY ] = isset( $instance[ self::PARENT_CATEGORY ] ) ? $instance[ self::PARENT_CATEGORY ] : 0;

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::NAME, 'parent-category-wrap' ); ?>
				<label>
					<?php esc_html_e( 'Display widget on every page', 'advanced-sidebar-menu' ); ?>
				</label>
				<br />
				<span class="description">
					<em>
						<?php esc_html_e( 'Display the same menu on every page that has this widget area, including the homepage if applicable.', 'advanced-sidebar-menu' ); ?>
					</em>
				</span>
			</p>
			<div <?php $widget->hide_element( self::NAME, 'parent-category-wrap' ); ?>>
				<p>
					<label for="<?php echo esc_attr( $widget->get_field_id( self::PARENT_CATEGORY ) ); ?>">
						<?php
						/* translators: Selected post type singular label */
						printf( esc_html__( 'Highest level parent %s', 'advanced-sidebar-menu' ), esc_html( strtolower( Taxonomy::instance()->get_taxonomy_label( $instance ) ) ) );
						?>
					</label>
					<?php
					wp_dropdown_categories( [
						'taxonomy'        => Taxonomy::instance()->get_taxonomy( 'category', [], $instance ),
						'name'            => $widget->get_field_name( self::PARENT_CATEGORY ),
						'id'              => $widget->get_field_id( self::PARENT_CATEGORY ),
						'selected'        => $instance[ self::PARENT_CATEGORY ],
						'hierarchical'    => true,
						'depth'           => 1,
						/* translators: the plural name of taxonomy */
						'show_option_all' => sprintf( __( '-- All %s --', 'advanced-sidebar-menu' ), Taxonomy::instance()->get_taxonomy_label( $instance, false ) ),
						'class'           => 'advanced-sidebar-menu-block-field',
					] ); //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					?>
					<span class="description">
						<em>
							<?php
							/* translators: Selected post type singular label */
							printf( esc_html__( 'Menu generates using this %s as top level item.', 'advanced-sidebar-menu' ), esc_html( strtolower( Taxonomy::instance()->get_taxonomy_label( $instance ) ) ) );
							?>
						</em>
					</span>
				</p>
			</div>
		</div>
		<?php
	}

}
