<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Category;

use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Taxonomy selection widget options.
 *
 * @since  2.4.0
 */
class Taxonomy extends Widget_Options_Abstract {
	use Singleton;

	const NAME = 'taxonomy';


	/**
	 * Actions and filters.
	 */
	protected function hook() {
		add_filter( 'advanced-sidebar-menu/menus/category/taxonomy', [ $this, 'get_taxonomy' ], 10, 3 );
		add_filter( 'advanced-sidebar-menu/widget/category/taxonomy-for-label', [ $this, 'get_taxonomy' ], 10, 3 );
		add_action( 'advanced-sidebar-menu/widget/category/right-column', [ $this, 'render_fields' ], 10, 2 );
	}


	/**
	 * Filter widget taxonomy using the widget settings.
	 *
	 * @param string $taxonomy - Current taxonomy (defaults to 'category' ).
	 * @param array  $args     - Widget registration args.
	 * @param array  $instance - Widget settings.
	 *
	 * @return mixed
	 */
	public function get_taxonomy( $taxonomy, array $args, array $instance ) {
		if ( ! empty( $instance[ self::NAME ] ) && taxonomy_exists( $instance[ self::NAME ] ) ) {
			return $instance[ self::NAME ];
		}

		return $taxonomy;
	}


	/**
	 * Get the label for use taxonomy.
	 *
	 * For adjusting widget option labels.
	 *
	 * @param array $instance - Widget settings.
	 * @param bool  $single   - Singular label or plural.
	 *
	 * @since 8.2.0
	 *
	 * @return mixed
	 */
	public function get_taxonomy_label( $instance, $single = true ) {
		$taxonomy = get_taxonomy( $this->get_taxonomy( 'category', [], $instance ) );
		if ( empty( $taxonomy ) ) {
			$taxonomy = get_taxonomy( 'category' ); // Sensible fallback.
		}

		return $single ? $taxonomy->labels->singular_name : $taxonomy->labels->name;
	}

	/**
	 * Get available taxonomies for the widget
	 *
	 * @return array
	 */
	public function get_taxonomies() {
		$taxonomies = [];
		$objects = get_taxonomies( [
			'hierarchical' => true,
			'_builtin'     => false,
		], 'objects' );
		foreach ( $objects as $_object ) {
			$taxonomies[ $_object->name ] = $_object->labels->name;
		}

		return apply_filters( 'advanced-sidebar-menu-pro/widget-options/taxonomy/taxonomies', $taxonomies );
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$taxonomies = $this->get_taxonomies();
		if ( empty( $taxonomies ) ) {
			return;
		}
		$instance[ self::NAME ] = isset( $instance[ self::NAME ] ) ? $instance[ self::NAME ] : '';
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<label for="<?php echo esc_attr( $widget->get_field_id( self::NAME ) ); ?>">
					<?php esc_html_e( 'Taxonomy', 'advanced-sidebar-menu' ); ?>
				</label>
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::NAME ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::NAME ) ); ?>">
					<option value="0">
						<?php esc_html_e( 'Categories', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					foreach ( $taxonomies as $_name => $_label ) {
						printf( '<option value="%s" %s">%s</option>', esc_attr( $_name ), selected( $_name, $instance[ self::NAME ] ), esc_html( $_label ) );
					}
					?>
				</select>
			</p>
		</div>
		<?php
	}
}
