<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Page;

use Advanced_Sidebar_Menu\List_Pages;
use Advanced_Sidebar_Menu\Menus\Page;
use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * "Display the current page's direct family only" and related widget options
 */
class Current_Page_Parent_Only extends Widget_Options_Abstract {
	use Singleton;

	const NAME = 'parent_only';

	const DISPLAY_SIBLINGS_CHILD_PAGE       = 'parent_only/display-sibling/child-page';
	const DISPLAY_SIBLINGS_NO_GRANDCHILDREN = 'parent_only/display-sibling/no-grandchildren';


	/**
	 * Actions and filters.
	 */
	protected function hook() {
		add_filter( 'advanced-sidebar-menu/list-pages/first-level-child-pages', [
			$this,
			'exclude_non_ancestor_pages',
		], 1, 3 );
		add_filter( 'advanced-sidebar-menu/list-pages/grandchild-pages', [
			$this,
			'exclude_non_ancestor_pages',
		], 1, 3 );

		add_action( 'advanced-sidebar-menu/widget/page/left-column', [ $this, 'render_fields' ], 15, 2 );
	}


	/**
	 * Exclude any non ancestor child pages
	 *
	 * Removes any page which is not a parent, grandparent, child
	 * or grandchild of the current page.
	 *
	 * Will include siblings of the current page if the options
	 * are checked to do so.
	 *
	 * @param \WP_Post[] $pages      - List of current pages pre filter.
	 * @param List_Pages $list_pages - List_Pages class.
	 * @param Page       $menu       - Menu class.
	 *
	 * @return array
	 */
	public function exclude_non_ancestor_pages( array $pages, List_Pages $list_pages, Page $menu ) {
		$current_page = $menu->get_current_post();
		if ( null === $current_page || 0 === $current_page->post_parent || ! $this->checked( self::NAME, $menu->instance ) ) {
			return $pages;
		}

		if ( in_array( $current_page->ID, wp_list_pluck( $pages, 'ID' ), true ) ) {
			// Display siblings of the current page.
			if ( $this->checked( self::DISPLAY_SIBLINGS_CHILD_PAGE, $menu->instance ) ) {
				return $pages;
			}

			// Display siblings when the current page has no children.
			if ( $this->checked( self::DISPLAY_SIBLINGS_NO_GRANDCHILDREN, $menu->instance ) ) {
				$children = $list_pages->get_child_pages( $current_page->ID );
				if ( empty( $children ) ) {
					return $pages;
				}
			}
		}

		// Default behavior with no sub options checked.
		foreach ( $pages as $k => $_post ) {
			if ( ! $list_pages->is_current_page_ancestor( $_post->ID ) && ! \in_array( $current_page->ID, \get_post_ancestors( $_post ), true ) ) {
				unset( $pages[ $k ] );
			}
		}

		return $pages;
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget Class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance[ self::NAME ] = isset( $instance[ self::NAME ] ) ? $instance[ self::NAME ] : '';

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::NAME, self::DISPLAY_SIBLINGS_CHILD_PAGE ); ?>
				<label>
					<?php
					/* translators: Selected post type single label */
					printf( esc_html__( "Display the current %s's direct family only", 'advanced-sidebar-menu' ), esc_html( strtolower( Post_Type::instance()->get_post_type_label( $instance ) ) ) );
					?>
				</label>
				<br />
				<span class="description">
					<em>
						<?php
						/* translators: Selected post type single label */
						printf( esc_html__( 'Only display the current %1$s and any parents, grandparents, children or grandchildren of the current %1$s.', 'advanced-sidebar-menu' ), esc_html( strtolower( Post_Type::instance()->get_post_type_label( $instance ) ) ) );
						?>
					</em>
				</span>
			</p>
			<div <?php $widget->hide_element( self::NAME, self::DISPLAY_SIBLINGS_CHILD_PAGE ); ?>>

				<p>
					<?php $widget->checkbox( self::DISPLAY_SIBLINGS_CHILD_PAGE, self::DISPLAY_SIBLINGS_NO_GRANDCHILDREN ); ?>
					<label>
						<?php
						/* translators: Selected post type single label */
						printf( esc_html__( 'Display siblings of the current %s', 'advanced-sidebar-menu' ), esc_html( strtolower( Post_Type::instance()->get_post_type_label( $instance ) ) ) );
						?>
					</label>
					<br />
					<span class="description">
						<em>
							<?php
							/* translators: Selected post type single label */
							printf( esc_html__( 'Display siblings of the current %1$s when on a child or grandchild %1$s.', 'advanced-sidebar-menu' ), esc_html( strtolower( Post_Type::instance()->get_post_type_label( $instance ) ) ) );
							?>
						</em>
					</span>
				</p>
				<p <?php $widget->hide_element( self::DISPLAY_SIBLINGS_CHILD_PAGE, self::DISPLAY_SIBLINGS_NO_GRANDCHILDREN, true ); ?>>
					<?php $widget->checkbox( self::DISPLAY_SIBLINGS_NO_GRANDCHILDREN ); ?>
					<label>
						<?php esc_html_e( 'Display siblings when there are no children', 'advanced-sidebar-menu' ); ?>
					</label>
					<br />
					<span class="description">
						<em>
							<?php
							/* translators: Selected post type single label */
							printf( esc_html__( 'Display siblings of the current %1$s when the current %1$s has no children.', 'advanced-sidebar-menu' ), esc_html( strtolower( Post_Type::instance()->get_post_type_label( $instance ) ) ) );
							?>
						</em>
					</span>
				</p>
			</div>
		</div>
		<?php
	}

}
