<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Page;

use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Display Everywhere Option for Page Widget
 *
 * @author OnPoint Plugins
 * @since  3.1.0
 */
class Display_Everywhere extends Widget_Options_Abstract {
	use Singleton;

	const NAME = 'display_everywhere';

	const PARENT_PAGE = 'display_everywhere/parent_page';


	/**
	 * Actions and filters.
	 */
	protected function hook() {
		add_filter( 'advanced-sidebar-menu/menus/page/top-parent', [ $this, 'get_top_id_from_setting' ], 10, 3 );
		add_filter( 'advanced-sidebar-menu/menus/page/is-displayed', [ $this, 'display_always' ], 10, 3 );

		add_action( 'advanced-sidebar-menu/widget/page/right-column', [ $this, 'render_fields' ], 5, 2 );
	}


	/**
	 * Get the top id from the current widget's settings.
	 *
	 * @param int   $top_id      - Native menu's top id.
	 * @param array $widget_args - Widget class arguments.
	 * @param array $instance    - Widget settings.
	 *
	 * @since 3.11.2
	 *
	 * @return int
	 */
	public function get_top_id_from_setting( $top_id, array $widget_args, array $instance ) {
		if ( $this->checked( self::NAME, $instance ) && ! empty( $instance[ self::PARENT_PAGE ] ) ) {
			return $instance[ self::PARENT_PAGE ];
		}

		return $top_id;
	}


	/**
	 * Make sure this menu will always display if this option is checked.
	 *
	 * @param bool  $displayed   - Native menu's determination if is displayed.
	 * @param array $widget_args - Widget class arguments.
	 * @param array $instance    - Widget settings.
	 *
	 * @return bool
	 */
	public function display_always( $displayed, array $widget_args, array $instance ) {
		if ( $this->checked( self::NAME, $instance ) ) {
			return true;
		}

		return $displayed;
	}


	/**
	 * Get the label of the selected post type.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @return mixed
	 *
	 * @deprecated
	 */
	public function get_post_type_label( array $instance ) {
		_deprecated_function( __METHOD__, '8.2.0', esc_html( Post_Type::class . '::get_post_type_label' ) );

		return Post_Type::instance()->get_post_type_label( $instance );
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance[ self::NAME ] = isset( $instance[ self::NAME ] ) ? $instance[ self::NAME ] : '';
		$instance[ self::PARENT_PAGE ] = isset( $instance[ self::PARENT_PAGE ] ) ? $instance[ self::PARENT_PAGE ] : 0;
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::NAME, 'top-level-wrap' ); ?>
				<label>
					<?php esc_html_e( 'Display widget on every page', 'advanced-sidebar-menu' ); ?>
				</label>
				<br />
				<span class="description">
					<em><?php esc_html_e( 'Display the same menu on every page that has this widget area, including the homepage if applicable.', 'advanced-sidebar-menu' ); ?></em>
				</span>
			</p>
			<div <?php $widget->hide_element( self::NAME, 'top-level-wrap' ); ?>>
				<p>
					<label for="<?php echo esc_attr( $widget->get_field_id( self::PARENT_PAGE ) ); ?>">
						<?php
						/* translators: Selected post type singular label */
						printf( esc_html__( 'Highest level parent %s', 'advanced-sidebar-menu' ), esc_html( strtolower( Post_Type::instance()->get_post_type_label( $instance ) ) ) );
						?>
					</label>
					<?php
					//phpcs:disable WordPress.Security.EscapeOutput.OutputNotEscaped
					wp_dropdown_pages( [
						'post_type' => Post_Type::instance()->get_post_type_from_option( 'page', [], $instance ),
						'name'      => $widget->get_field_name( self::PARENT_PAGE ),
						'id'        => $widget->get_field_id( self::PARENT_PAGE ),
						'selected'  => $instance[ self::PARENT_PAGE ],
						'depth'     => 1,
						'class'     => 'advanced-sidebar-menu-block-field',
					] );
					//phpcs:enable WordPress.Security.EscapeOutput.OutputNotEscaped
					?>
					<span class="description">
						<em>
							<?php
							/* translators: Selected post type singular label */
							printf( esc_html__( 'Menu generates using this %s as the top level item.', 'advanced-sidebar-menu' ), esc_html( strtolower( Post_Type::instance()->get_post_type_label( $instance ) ) ) );
							?>
						</em>
					</span>
				</p>
			</div>
		</div>

		<?php
	}
}
