<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Page;

use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Post type widget option.
 *
 * @author OnPoint Plugins
 * @since  2.3.0
 */
class Post_Type extends Widget_Options_Abstract {
	use Singleton;

	const NAME = 'post_type';


	/**
	 * Actions and filters.
	 */
	protected function hook() {
		add_filter( 'advanced-sidebar-menu/menus/page/post-type', [ $this, 'get_post_type_from_option' ], 10, 3 );
		add_filter( 'advanced-sidebar-menu/widget/page/post-type-for-label', [ $this, 'get_post_type_from_option' ], 10, 3 );
		add_action( 'advanced-sidebar-menu/widget/page/right-column', [ $this, 'render_fields' ], 10, 2 );
	}


	/**
	 * Filter the widget's post type via the current widget's option.
	 *
	 * @param string $post_type   - Provided post type.
	 * @param array  $widget_args - Widget class arguments. (not used here).
	 * @param array  $instance    - Widget settings.
	 *
	 * @return mixed
	 */
	public function get_post_type_from_option( $post_type, $widget_args, array $instance ) {
		if ( ! empty( $instance[ self::NAME ] ) ) {
			return $instance[ self::NAME ];
		}

		return $post_type;
	}


	/**
	 * Get the label of the selected post type.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @param bool  $single   - Singular or plural version of label.
	 *
	 * @since 8.2.0
	 *
	 * @return mixed
	 */
	public function get_post_type_label( array $instance, $single = true ) {
		$post_type = get_post_type_object( $this->get_post_type_from_option( 'page', [], $instance ) );

		if ( null === $post_type ) {
			$post_type = get_post_type_object( 'page' ); // Sensible fallback.
		}

		return $single ? $post_type->labels->singular_name : $post_type->labels->name;
	}


	/**
	 * Get available post types for this widget
	 *
	 * @access public for unit testing
	 *
	 * @return array
	 */
	public function get_post_types() {
		$post_types = [
			'page' => _x( 'Pages', 'post type general name' ), //phpcs:ignore WordPress.WP.I18n.MissingArgDomain
		];
		$objects = get_post_types( [
			'_builtin'     => false,
			'hierarchical' => true,
			'public'       => true,
		], 'objects' );
		foreach ( $objects as $_object ) {
			$post_types[ $_object->name ] = $_object->labels->name;
		}

		return apply_filters( 'advanced-sidebar-menu-pro/widget-options/page/post-type/available-post-types', $post_types );
	}


	/**
	 * Render the widget options.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$post_types = $this->get_post_types();
		unset( $post_types['page'] ); // Handled separately.
		if ( empty( $post_types ) ) {
			return;
		}
		$instance[ self::NAME ] = isset( $instance[ self::NAME ] ) ? $instance[ self::NAME ] : '';
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<label for="<?php echo esc_attr( $widget->get_field_id( self::NAME ) ); ?>">
					<?php esc_html_e( 'Post type', 'advanced-sidebar-menu' ); ?>
				</label>
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::NAME ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::NAME ) ); ?>">
					<option value="0">
						<?php esc_html_e( 'Pages', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					foreach ( $post_types as $_name => $_label ) {
						printf( '<option value="%s" %s">%s</option>', esc_attr( $_name ), selected( $_name, $instance[ self::NAME ] ), esc_html( $_label ) );
					}
					?>
				</select>
			</p>
		</div>
		<?php
	}

}
