<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Shared;

use Advanced_Sidebar_Menu\Menus\Menu_Abstract;
use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Navigation_Menu;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Accordion widget settings.
 *
 * @author OnPoint Plugins
 */
class Accordion extends Widget_Options_Abstract {
	use Singleton;

	const NAME = 'enable_accordion';

	const ENABLE_ACCORDION    = self::NAME;
	const ACCORDION_STYLE     = 'accordion_style';
	const ICON_COLOR          = 'icon_color';
	const KEEP_CLOSED         = 'keep_closed';
	const INCLUDE_PARENT      = 'include_parent_in_accordion';
	const LINKS_EXPAND        = 'links_expand';
	const LINKS_EXPAND_LEVELS = 'links_expand_levels';

	/**
	 * Default widget settings.
	 *
	 * @var array
	 */
	protected $defaults = [
		self::ENABLE_ACCORDION        => 0,
		self::ACCORDION_STYLE         => 'plus',
		self::ICON_COLOR              => false,
		self::KEEP_CLOSED             => false,
		Menu_Abstract::INCLUDE_PARENT => false,
		self::INCLUDE_PARENT          => false,
		self::LINKS_EXPAND            => false,
		self::LINKS_EXPAND_LEVELS     => [
			'all' => 'checked',
		],
	];


	/**
	 * Actions.
	 */
	protected function hook() {
		add_action( 'advanced-sidebar-menu/menus/page/render', [ $this, 'render' ] );
		add_action( 'advanced-sidebar-menu/menus/category/render', [ $this, 'render' ] );
		add_action( 'advanced-sidebar-menu/menus/navigation/render', [ $this, 'render' ] );

		add_action( 'advanced-sidebar-menu/widget/page/right-column', [ $this, 'render_fields' ], 20, 2 );
		add_action( 'advanced-sidebar-menu/widget/category/right-column', [ $this, 'render_fields' ], 20, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/right-column', [ $this, 'render_fields' ], 10, 2 );
	}


	/**
	 * Render the required js and css on the front end.
	 *
	 * @param Menu_Abstract $menu - menu class.
	 *
	 * @return void
	 */
	public function render( Menu_Abstract $menu ) {
		$instance = $menu->get_widget_instance();
		$widget_args = $menu->get_widget_args();
		if ( empty( $instance[ self::NAME ] ) ) {
			return;
		}

		wp_enqueue_style( 'dashicons' );
		wp_enqueue_script( 'advanced-sidebar-menu/widget-options/shared/accordion', ADVANCED_SIDEBAR_MENU_PRO_URL . 'assets/js/advanced-sidebar-menu-accordions.js', [ 'jquery' ], ADVANCED_SIDEBAR_MENU_PRO_VERSION, true );
		wp_localize_script( 'advanced-sidebar-menu/widget-options/shared/accordion', 'ADVANCED_SIDEBAR_MENU_ACCORDIONS', $this->js_config() );

		?>
		<style>
			#<?php echo esc_attr( $widget_args['widget_id'] ); ?>-wrap .dashicons {
				cursor: pointer;
			<?php
			echo empty( $instance[ Style_Targeting::BLOCK_STYLE ] ) ? '' : 'margin: 2px 5px 0;';
			?>
			}

			#<?php echo esc_attr( $widget_args['widget_id'] ); ?>-wrap .dashicons:not(:focus-visible) {
				outline: none;
			}
		</style>
		<?php

		if ( ! empty( $instance[ self::ICON_COLOR ] ) ) {
			?>
			<style>
				#<?php echo esc_attr( $widget_args['widget_id'] ); ?>-wrap .dashicons {
					color: <?php echo esc_attr( $instance[ self::ICON_COLOR ] ); ?> !important;
				}
			</style>
			<?php
		}

		$this->inline_js( $menu );
	}


	/**
	 * Register the accordion via inline JS.
	 *
	 * Done this way to account for Page Builders like Elementor.
	 *
	 * @param Menu_Abstract $menu - Current menu being processed.
	 */
	protected function inline_js( Menu_Abstract $menu ) {
		$settings = array_merge( $this->defaults, array_intersect_key( $menu->get_widget_instance(), $this->defaults ) );
		$settings[ static::LINKS_EXPAND_LEVELS ] = array_keys( $settings[ static::LINKS_EXPAND_LEVELS ] );
		$widget_args = $menu->get_widget_args();
		?>
		<script>
			var asm_pro = window.asm_pro || {};
			if ( 'undefined' === typeof asm_pro.enabled_accordions ) {
				asm_pro.enabled_accordions = {};
			}
			asm_pro.enabled_accordions[ "<?php echo esc_js( $widget_args['widget_id'] ) . '-wrap"] = ' . wp_json_encode( $settings ); ?>;
		</script>
		<?php
	}


	/**
	 * JavaScript configuration to pass to the accordions.
	 *
	 * @return array
	 */
	public function js_config() {
		return [
			'icons'     => $this->get_accordion_style_options(),
			'iconStyle' => static::ACCORDION_STYLE,
		];
	}


	/**
	 * Get available icon sets which will be used by accordion menus.
	 *
	 * @since 3.12.0 (Support screen readers tabbing and entering)
	 *
	 * @return array
	 */
	public function get_accordion_style_options() {
		$options = [
			'plus'        => [
				'label'    => __( 'Plus', 'advanced-sidebar-menu' ),
				'expand'   => '<span aria-expanded="false" tabindex="0" role="button" class="dashicons dashicons-plus"></span>',
				'contract' => '<span aria-expanded="true" tabindex="0" role="button" class="dashicons dashicons-minus"></span>',
			],
			'arrow'       => [
				'label'    => __( 'Arrow', 'advanced-sidebar-menu' ),
				'expand'   => '<span aria-expanded="false" tabindex="0" role="button" class="dashicons dashicons-arrow-down-alt2"></span>',
				'contract' => '<span aria-expanded="true" tabindex="0" role="button" class="dashicons dashicons-arrow-up-alt2"></span>',
			],
			'solid_arrow' => [
				'label'    => __( 'Solid Arrow', 'advanced-sidebar-menu' ),
				'expand'   => '<span aria-expanded="false" tabindex="0" role="button" class="dashicons dashicons-arrow-down"></span>',
				'contract' => '<span aria-expanded="true" tabindex="0" role="button" class="dashicons dashicons-arrow-up"></span>',
			],
			'hamburger'   => [
				'label'    => __( 'Hamburger', 'advanced-sidebar-menu' ),
				'expand'   => '<span aria-expanded="false" tabindex="0" role="button" class="dashicons dashicons-menu"></span>',
				'contract' => '<span aria-expanded="true" tabindex="0" role="button" class="dashicons dashicons-minus"></span>',
			],
		];

		return apply_filters( 'advanced-sidebar-menu/styles/accordion', $options );
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - widget settings.
	 * @param Widget_Abstract $widget   - widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance = (array) wp_parse_args( $instance, $this->defaults );
		$widget->set_instance( $instance, [] );
		$style_options = $this->get_accordion_style_options();

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::ENABLE_ACCORDION, 'accordion-wrap' ); ?>
				<label>
					<?php esc_html_e( 'Enable accordion menu', 'advanced-sidebar-menu' ); ?>
				</label>
				<?php
				if ( Navigation_Menu::NAME !== $widget->id_base ) {
					?>
					<br />
					<span class="description">
						<em>
							<?php
							/* translators: Selected post type plural label */
							printf( esc_html__( 'Commonly used with "Always display child %s."', 'advanced-sidebar-menu' ), esc_html( strtolower( $this->get_object_type_label( $instance, $widget, false ) ) ) );
							?>
						</em>
					</span>
					<?php
				}
				?>
			</p>
			<div <?php $widget->hide_element( self::ENABLE_ACCORDION, 'accordion-wrap' ); ?>>
				<p>
					<label>
						<?php esc_html_e( 'Icon style', 'advanced-sidebar-menu' ); ?>
					</label>
					<select
						data-js="advanced-sidebar-menu-pro/widget/accordion-style"
						data-target="<?php echo esc_attr( $widget->get_field_id( 'accordion-icons' ) ); ?>"
						id="<?php echo esc_attr( $widget->get_field_id( self::ACCORDION_STYLE ) ); ?>"
						name="<?php echo esc_attr( $widget->get_field_name( self::ACCORDION_STYLE ) ); ?>"
						title="<?php esc_attr_e( 'Accordion icons style', 'advanced-sidebar-menu' ); ?>"
						style="margin-left: 3px; width:103px !important;"
					>
						<?php
						foreach ( $style_options as $_style => $_icons ) {
							printf( '<option value="%s" %s>%s</option>', esc_attr( $_style ), selected( $_style, $instance[ self::ACCORDION_STYLE ] ), esc_html( $_icons['label'] ) );
						}
						?>
					</select>
					<span
						data-js="<?php echo esc_attr( $widget->get_field_id( 'accordion-icons' ) ); ?>"
						class="advanced-sidebar-menu-icon-preview"
						style="<?php echo empty( $instance[ self::ICON_COLOR ] ) ? '' : esc_attr( "color: {$instance[ self::ICON_COLOR ]}" ); ?>">
						<?php
						echo wp_kses( $style_options[ $instance[ self::ACCORDION_STYLE ] ]['contract'] . ' ' . $style_options[ $instance[ self::ACCORDION_STYLE ] ]['expand'], [ 'span' => [ 'class' => [] ] ] );
						?>
					</span>
				</p>
				<p>
					<label for="<?php echo esc_attr( $widget->get_field_id( self::ICON_COLOR ) ); ?>">
						<?php esc_html_e( 'Icon color', 'advanced-sidebar-menu' ); ?>
					</label>
					<input
						data-js="advanced-sidebar-menu-pro/widget/accordion-color"
						data-target="<?php echo esc_attr( $widget->get_field_id( 'accordion-icons' ) ); ?>"
						type="text"
						name="<?php echo esc_attr( $widget->get_field_name( self::ICON_COLOR ) ); ?>"
						value="<?php echo esc_attr( $instance[ self::ICON_COLOR ] ); ?>"
						id="<?php echo esc_attr( $widget->get_field_id( self::ICON_COLOR ) ); ?>"
						class="asm-color-select">
					<br />
				</p>

				<p>
					<?php $widget->checkbox( self::KEEP_CLOSED ); ?>

					<label>
						<?php esc_html_e( 'Close all accordion sections until clicked', 'advanced-sidebar-menu' ); ?>
					</label>
					<br />
					<span class="description">
						<em><?php esc_html_e( 'If unchecked, the menu will open to the current section automatically.', 'advanced-sidebar-menu' ); ?></em>
					</span>
				</p>
				<p>
					<?php $widget->checkbox( self::INCLUDE_PARENT ); ?>
					<label>
						<?php esc_html_e( 'Include highest level parent in accordion', 'advanced-sidebar-menu' ); ?>
					</label>
					<br />
					<span class="description">
						<em><?php esc_html_e( 'If checked, the highest level parent will become part of the accordion open/close ability.', 'advanced-sidebar-menu' ); ?></em>
					</span>
				</p>
				<p>
					<?php $widget->checkbox( self::LINKS_EXPAND, 'links-expand-levels' ); ?>
					<label>
						<?php esc_html_e( 'Use links for open/close', 'advanced-sidebar-menu' ); ?>
					</label>
					<br />
					<span class="description">
						<em><?php esc_html_e( 'If checked, links of the selected levels will open and close their section when clicked.', 'advanced-sidebar-menu' ); ?></em>
					</span>
				</p>
				<div <?php $widget->hide_element( self::LINKS_EXPAND, 'links-expand-levels' ); ?>>
					<p>
						<label>
							<?php esc_html_e( 'Levels of links to use for open/close', 'advanced-sidebar-menu' ); ?>:
						</label>
						<span class="description">
							<em><?php esc_html_e( 'Unchecked levels will only open and close when the icons are clicked.', 'advanced-sidebar-menu' ); ?></em>
						</span>
						<label>
							<?php
							$widget->checkbox( self::LINKS_EXPAND_LEVELS . '[all]', 'links-expand-all' );
							esc_html_e( 'All levels', 'advanced-sidebar-menu' );
							?>
						</label>

						<span <?php $widget->hide_element( self::LINKS_EXPAND_LEVELS . '[all]', 'links-expand-all', true ); ?>>
							<br />
							<label>
								<?php
								$widget->checkbox( self::LINKS_EXPAND_LEVELS . '[highest]' );
								esc_html_e( 'Highest level parent links', 'advanced-sidebar-menu' );
								?>
							</label>
							<br />
							<label>
								<?php
								$widget->checkbox( self::LINKS_EXPAND_LEVELS . '[child]' );
								esc_html_e( 'Child links', 'advanced-sidebar-menu' );
								?>
							</label>
							<br />
							<label>
								<?php
								$widget->checkbox( self::LINKS_EXPAND_LEVELS . '[grandchild]' );
								esc_html_e( 'Grandchild links', 'advanced-sidebar-menu' );
								?>
							</label>
						</span>
					</p>
				</div>
			</div>
		</div>
		<?php
	}

}
